/*
    Copyright (C) 1998, 1999, 2000  Charles Mac Donald

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "shared.h"

/* Global data */
int input_mask;              /* Holds state of emulated buttons */
t_bitmap bitmap;             /* Memory bitmap or video RAM for rendering */
t_cart cart;
t_snd snd;

void system_init(int rate)
{
    /* Initialize the VDP emulation */
    vdp_init();

    /* Initialize the SMS emulation */
    sms_init();

    /* Initialize the look-up tables and related data */
    render_init();

    /* Enable sound emulation if a sample rate was specified */
    if(rate)
    {
        /* FIXME: don't hard-code FPS value for PAL emulation */
        snd.bufsize = (rate / 60);

        /* Allocate enough memory for the PSG emulator output */
        snd.buffer = (signed short int *)malloc(snd.bufsize * 2);

        /* Set up SN76489 emulation */
        SN76496_init(0, PSG_CLOCK, 255, rate);
    }
    else
    {
        /* Initialize sound related data */
        snd.bufsize = 0;
        snd.buffer = 0;
    }

    /* Don't save SRAM by default */
    sms.save = 0;

    /* Clear emulated button state */
    input_mask = 0;
}


void system_reset(void)
{
    vdp_reset();
    sms_reset();
    render_reset();
}




