
#ifndef _SP_H_
#define _SP_H_

/* Image types */
#define TYPE_SMS            (0)
#define TYPE_GG             (1)

/* SN76489 clock speed */
#define PSG_CLOCK           (3579545)

/* Pixel mask */
#define PIXEL_MASK          (0x1F)

/* Bit definitions for the input_mask variable */
#define KEY_PAD1_UP         (0x00000001)
#define KEY_PAD1_DOWN       (0x00000002)
#define KEY_PAD1_LEFT       (0x00000004)
#define KEY_PAD1_RIGHT      (0x00000008)
#define KEY_PAD1_BUTTON2    (0x00000010)
#define KEY_PAD1_BUTTON1    (0x00000020)
#define KEY_PAD2_UP         (0x00000040)    /* SMS Only */
#define KEY_PAD2_DOWN       (0x00000080)    /* SMS Only */
#define KEY_PAD2_LEFT       (0x00000100)    /* SMS Only */
#define KEY_PAD2_RIGHT      (0x00000200)    /* SMS Only */
#define KEY_PAD2_BUTTON2    (0x00000400)    /* SMS Only */
#define KEY_PAD2_BUTTON1    (0x00000800)    /* SMS Only */
#define KEY_PAD_START       (0x00001000)    /* GG Only */
#define KEY_PAD_RESET       (0x00002000)    /* SMS Only - Hardware reset for GG */
#define KEY_PAD_PAUSE       (0x00004000)    /* Fake key - causes NMI (SMS Only) */
#define KEY_EXIT            (0x00008000)    /* Fake key - quits emulator */

/* Sound emulation structure */
typedef struct
{
    int bufsize;
    signed short *buffer;
}t_snd;

/* Game image structure */
typedef struct
{
    byte *rom;              /* Pointer to ROM data */
    byte pages;             /* Number of 16k pages */
    byte type;              /* Image type (GG or SMS) */
}t_cart;

/* Bitmap structure */
typedef struct
{
    unsigned char *data;    /* Pointer to bitmap data */
    int width;              /* Width of bitmap in pixels */
    int height;             /* Height of bitmap in lines */
    int pitch;              /* Width of bitmap in bytes */
    int depth;              /* Pixel color depth (8, 15, 16, 24, 32) */
    struct
    {
        byte color[32][3];  /* Color data, R, G, B, 8-bits */
        byte dirty[32];     /* Dirty flag */
        byte update;        /* Global color update flag */
    }pal;
}t_bitmap;

/* Global variables */
extern t_bitmap bitmap;
extern t_snd snd;
extern t_cart cart;
extern int input_mask;

/* Function prototypes */
void system_init(int sam_rate);
void system_reset(void);

#endif /* _SP_H_ */
